package  
{
	import base.Assets;
	import base.GameEntity;
	import flash.geom.Point;
	import net.flashpunk.graphics.Tilemap;
	import net.flashpunk.masks.Grid;

	public final class Path extends GameEntity 
	{
		private var _tileset:Tilemap;
		private var _grid:Grid;
		private var _path:Vector.<Point>;
		
		public function Path(tileset:Class, path:Vector.<Point>) 
		{
			_init(tileset);
			_path = path;
			_drawPath();
		}
		
		private function _init(tiles:Class):void
		{
			_tileset = new Tilemap(tiles, 800, 480, 32, 32);
			_grid = new Grid(800, 480, 32, 32);
			graphic = _tileset;
		}
		
		private function _drawPath():void
		{
			var i:int, sx:int, sy:int, w:int, h:int;
			
			for (i = 0; i < _path.length - 1; i++)
			{
				sx = Math.min(_path[i].x, _path[i + 1].x);
				sy = Math.min(_path[i].y, _path[i + 1].y);
				w = Math.max(Math.abs(_path[i].x - _path[i + 1].x) + 1, 1);
				h = Math.max(Math.abs(_path[i].y - _path[i + 1].y) + 1, 1);
				
				if (w == 1)
				{
					_tileset.setRect(sx, sy, w, h, 0);
					_grid.setRect(sx, sy, w, h, true);
				}
				else 
				{
					_tileset.setRect(sx, sy, w, h, 1);
					_grid.setRect(sx, sy, w, h, true);
				}
				
				
				if (i >= 1 && _path[i].x < _path[i + 1].x && _path[i - 1].y > _path[i].y)
					_tileset.setTile(_path[i].x, _path[i].y, 3);
				else if (i >= 1 && _path[i].x < _path[i + 1].x && _path[i - 1].y < _path[i].y)
					_tileset.setTile(_path[i].x, _path[i].y, 2);
				else if (i >= 1 && _path[i - 1].y < _path[i].y && _path[i].x > _path[i + 1].x)
					_tileset.setTile(_path[i].x, _path[i].y, 5);
				else if (i >= 1 && _path[i - 1].y > _path[i].y && _path[i].x > _path[i + 1].x)
					_tileset.setTile(_path[i].x, _path[i].y, 4);
				else if (i >= 1 && _path[i - 1].x > _path[i].x && _path[i].y < _path[i + 1].y)
					_tileset.setTile(_path[i].x, _path[i].y, 3)
				else if (i >= 1 && _path[i - 1].x > _path[i].x && _path[i].y > _path[i + 1].y)
					_tileset.setTile(_path[i].x, _path[i].y, 2)
				else if (_path[i].y > _path[i + 1].y)
					_tileset.setTile(_path[i].x, _path[i].y, 5);
				else if (_path[i].y < _path[i + 1].y)
					_tileset.setTile(_path[i].x, _path[i].y, 4);
					
			}
		}
		
		public function GetNextDestination(current:int):Point
		{
			if (current < _path.length - 1)
				return _path[current + 1];
			else
				return new Point( -1, -1);
		}
		
		public function GetEntryPoint():Point
		{
			return _path[0];
		}
		
		public function OnPath(xTile:int, yTile:int):Boolean
		{
			return _grid.getTile(xTile, yTile);
		}
	}
}